require 'readline'

# Questions and answers module.
#
# Example:
#
#   quna = Quna::Query.new( :yes )
#   ans = quna.ask( "Ready to scan" )
#   do_scan if ans
#
module Quna

    class QunaInterrupt < RuntimeError; end
    class QunaNoAnswer  < RuntimeError; end


    # Basic user query.
    class Query

        def initialize( default_answer = nil )
            @question = nil
            @default_answer = default_answer
            @answer = nil

            @opts = {
                no_answer_limit: nil,
            }
        end


        # Store asked question.
        def set_question( question )
            @question = question
        end


        # Ask y/n question until answer is received.
        def ask_yn( question )
            set_question question

            result = :no

            no_answer_count = 0

            loop do
                answer = Readline.readline( prompt_yn )
                result = case answer

                         when nil
                             raise( QunaInterrupt, @question )

                         when ""
                             if @default_answer
                                 @default_answer
                             else
                                 no_answer_count += 1
                                 if @opts[ :no_answer_limit ] && no_answer_count >= @opts[ :no_answer_limit ]
                                     raise( QunaNoAnswer, @question )
                                 end
                                 nil
                             end

                         else
                             lowcase = answer.downcase
                             case lowcase
                             when 'y', 'yes'; :yes
                             when 'n', 'no'; :no
                             else nil
                             end
                         end

                break if result
            end

            result
        end


        # Ask y/n question and return boolean result, i.e. TRUE for
        # yes and FALSE for no.
        def ask_yn_bool( question )
            result = ask_yn( question )
            if result == :yes
                true
            else
                false
            end
        end


        # Ask generic question.
        def ask( question )
            set_question question
            answer = Readline.readline( prompt )
            raise( QunaInterrupt, prompt ) unless answer
            answer
        end


        # Ask generic question without any requirements for answer.
        def ask_relaxed( question )
            set_question question
            answer = Readline.readline( prompt )
            answer
        end


        # Create prompt string for y/n question.
        def prompt_yn
            options = nil
            if @default_answer == :yes
                options = "[Y/n]"
            elsif @default_answer == :no
                options = "[y/N]"
            else
                options = "[y/n]"
            end
            "#{@question} #{options}? "
        end


        # Create prompt string for generic question.
        def prompt
            "#{@question}? "
        end

    end

end
